"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.skipWhileAsync = void 0;
const fromAsync_1 = require("../../async/static/fromAsync");
/**
 * Bypasses elements in a sequence as long as a specified condition is true and then returns the remaining elements.
 * The element's index is used in the logic of the predicate function.
 * @param source An Iterable<T> to return elements from.
 * @param predicate A function to test each source element for a condition;
 * the second parameter of the function represents the index of the source element.
 * @returns An IAsyncEnumerable<T> that contains the elements from the input sequence starting
 * at the first element in the linear series that does not pass the test specified by predicate.
 */
exports.skipWhileAsync = (source, predicate) => {
    if (predicate.length === 1) {
        return skipWhileAsync1(source, predicate);
    }
    else {
        return skipWhileAsync2(source, predicate);
    }
};
const skipWhileAsync1 = (source, predicate) => {
    async function* iterator() {
        let skip = true;
        for (const item of source) {
            if (skip === false) {
                yield item;
            }
            else if (await predicate(item) === false) {
                skip = false;
                yield item;
            }
        }
    }
    return fromAsync_1.fromAsync(iterator);
};
const skipWhileAsync2 = (source, predicate) => {
    async function* iterator() {
        let index = 0;
        let skip = true;
        for (const item of source) {
            if (skip === false) {
                yield item;
            }
            else if (await predicate(item, index) === false) {
                skip = false;
                yield item;
            }
            index++;
        }
    }
    return fromAsync_1.fromAsync(iterator);
};
